//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "uMain.h"
#include "uCity.h"
#include "Globe5Utils.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "GClasses"
#pragma link "Globe5"
#pragma link "GSpatialDatabase"
#pragma resource "*.dfm"
TfrmMain *frmMain;
TInterfaceList *WayPoints;

const int POZ_STEPS = 16;

//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner) : TForm(Owner)
{
}

//----------------------------------------------------------------------------
void __fastcall TfrmMain::ShowHint( TObject *Sender )
{
  pnlHint->Caption = " " + GetLongHint(Application->Hint);
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
  sdb->Active = true;
  WayPoints = new TInterfaceList();
  Application->OnHint = ShowHint;

  btnWorldClick( Sender );

  grdWayPoints->Cells[0][0] = "City";
  grdWayPoints->Cells[1][0] = "Statute Miles";
  grdWayPoints->Cells[2][0] = "Nautical Miles";
  grdWayPoints->Cells[3][0] = "Kilometers";
  grdWayPoints->Cells[0][1] = "     Total Distance";

  ctyLayer = Globe->GlobeLayer->LayerStore->LayerByName("TGCities");

  edtFilterChange( NULL );
  Globe->Projector->ScaleFactor = 0;
  FZoomOutSF = Globe->Projector->ScaleFactor;
  FZoomInSF = Globe->Projector->ScaleFactor * 8;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::edtFilterChange(TObject *Sender)
{
  lbxCities->Items->BeginUpdate();
  lbxCities->Items->Clear();
  AnsiString sTmp = Trim( edtFilter->Text );

  if ( ctyLayer != NULL )
  {
    _di_IGMapPoint obj = ctyLayer->ObjectSource->FirstObject();
    while ( obj != NULL )
    {
      if (( sTmp == "" ) || (CompareText(sTmp, obj->Attribute[0].SubString( 1, sTmp.Length())) == 0 ) )
        lbxCities->Items->AddObject(obj->Attribute[0],(TObject *)(obj->PrimaryKey));

      obj = ctyLayer->ObjectSource->NextObject( obj );
    }
  }

  lbxCities->Items->EndUpdate();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lbxCitiesDblClick(TObject *Sender)
{
  AddWayPoint( TGPrimaryKey( lbxCities->Items->Objects[lbxCities->ItemIndex] ));
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::grdWayPointsDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
  Accept = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::BuildWayPointList()
{
  grdWayPoints->RowCount = WayPoints->Count + 2;
  for( int idx = 0; idx < WayPoints->Count; ++idx )
    grdWayPoints->Cells[0][idx + 1] = static_cast<_di_IGMapPoint>( WayPoints->Items[idx] )->Attribute[0];

  grdWayPoints->Cells[0][grdWayPoints->RowCount - 1] = "   Total Distance";

  Extended eM = 0;
  Extended eNM = 0;
  Extended eK = 0;

  if ( WayPoints->Count > 0 )
  {
    TGPointLL ptFrom = static_cast<_di_IGMapPoint>( WayPoints->Items[0] )->Centroid;
    for ( int idx = 0; idx < WayPoints->Count; ++idx )
    {
      TGPointLL ptTo = static_cast<_di_IGMapPoint>( WayPoints->Items[idx] )->Centroid;
      int iTmp = DistanceLLToLL( ptFrom, ptTo, WGS84 );

      eM = eM + GUTo( iTmp, guMile );
      grdWayPoints->Cells[1][idx + 1] = FloatToStr( GUTo( iTmp, guMile ));

      eNM = eNM + GUTo( iTmp, guNauticalMile );
      grdWayPoints->Cells[2][idx + 1] = FloatToStr( GUTo( iTmp, guNauticalMile ));

      eK = eK + GUTo( iTmp, guKilometer );
      grdWayPoints->Cells[3][idx + 1] = FloatToStr( GUTo( iTmp, guKilometer ));

      ptFrom = ptTo;
    }

  }
  grdWayPoints->Cells[1][grdWayPoints->RowCount - 1] = FloatToStr( eM );
  grdWayPoints->Cells[2][grdWayPoints->RowCount - 1] = FloatToStr( eNM );
  grdWayPoints->Cells[3][grdWayPoints->RowCount - 1] = FloatToStr( eK );
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::AddWayPoint( TGPrimaryKey objPK )
{
  if ( ctyLayer != NULL )
  {
    WayPoints->Add( ctyLayer->ObjectSource->ByPrimaryKey( objPK ));
    BuildWayPointList();
    Globe->RedrawLayers();
  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::grdWayPointsDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
  AddWayPoint( reinterpret_cast<TGPrimaryKey>( lbxCities->Items->Objects[lbxCities->ItemIndex] ));
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::GlobeRender(TObject *Sender)
{
  Globe->Renderer->Pen->Width = 2;
  Globe->Renderer->Pen->Color = clRed;
  Globe->Renderer->Pen->Style = gpsSolid;

  if ( WayPoints->Count > 1 )
  {
    TGPointLL ptFrom = static_cast<_di_IGMapPoint>( WayPoints->Items[0] )->Centroid;
    for ( int idx = 1; idx < WayPoints->Count; ++idx)
    {
      TGPointLL ptTo = static_cast<_di_IGMapPoint>( WayPoints->Items[idx] )->Centroid;
      Globe->RenderGreatCircleLine( ptFrom, ptTo, gcShortest );
      ptFrom = ptTo;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::SpeedButton1Click(TObject *Sender)
{
  Globe->Projector->ProjectionClass = "TGSphericalPrj";
  btnWorldClick( Sender );

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::SpeedButton2Click(TObject *Sender)
{
  Globe->Projector->ProjectionClass = "TGMercatorPrj";
  btnWorldClick( Sender );
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnWorldClick(TObject *Sender)
{
  // reset the globe
  Globe->Projector->ScaleFactor = 0.0;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::grdWayPointsDblClick(TObject *Sender)
{
  if (( grdWayPoints->Selection.Top > 0 ) && ( grdWayPoints->Selection.Top < grdWayPoints->RowCount - 1 ))
  {
    WayPoints->Delete( grdWayPoints->Selection.Top - 1 );
    BuildWayPointList();
    Globe->RedrawLayers();
  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::About1Click(TObject *Sender)
{
//  frmAbout.ShowModal;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::Exit1Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnZoomInClick(TObject *Sender)
{
  Globe->Projector->ScaleFactor *= 2.0;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnZoomOutClick(TObject *Sender)
{
  Globe->Projector->ScaleFactor *= 0.5;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
  if ( Shift.Contains( ssCtrl ))
  {
    switch ( Key )
    {
      case VK_ADD:
        btnZoomInClick( NULL );
        break;
      case VK_SUBTRACT:
        btnZoomOutClick( NULL );
        break;
      default:
        return;
    }
    Key = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::PopupMenu1Popup(TObject *Sender)
{
  TPoint CursorPos;
  GetCursorPos(&CursorPos);

  lbxCities->ItemIndex = lbxCities->ItemAtPos(ScreenToClient(CursorPos), True);
  if ( lbxCities->ItemIndex == -1 )
    Abort();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::DeleteCity1Click(TObject *Sender)
{
  for( int idx = 0; idx < WayPoints->Count; ++idx )
    if ( static_cast<_di_IGMapPoint>( WayPoints->Items[idx] )->PrimaryKey == TGPrimaryKey(lbxCities->Items->Objects[lbxCities->ItemIndex]) )
    {
      WayPoints->Delete( idx );
      BuildWayPointList();
      break;
    }
  static_cast<_di_IGMapPoint>( ctyLayer->ObjectSource->ByPrimaryKey( TGPrimaryKey(lbxCities->Items->Objects[lbxCities->ItemIndex] )))->Delete();
  lbxCities->Items->Delete( lbxCities->ItemIndex );
  Globe->RedrawLayers();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::LocateCity1Click(TObject *Sender)
{
  if ( ctyLayer != NULL )
  {
    TGPointLL centroid = ctyLayer->ObjectSource->ByPrimaryKey( TGPrimaryKey(lbxCities->Items->Objects[lbxCities->ItemIndex] ))->Centroid;
    StartEasing( centroid.iLongX, centroid.iLatY );
  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::ModifyCity1Click(TObject *Sender)
{
  frmCityObject->ModifyCity(static_cast<_di_IGMapPoint>( ctyLayer->ObjectSource->ByPrimaryKey( TGPrimaryKey(lbxCities->Items->Objects[lbxCities->ItemIndex] ))));
  edtFilterChange(NULL);
  Globe->RedrawLayers();

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::AddCity1Click(TObject *Sender)
{
  frmCityObject->NewCity( ctyLayer );
  edtFilterChange(NULL);
  Globe->RedrawLayers();

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::GlobeMouseWheelDown(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  Globe->Projector->AdjustScaleFactor( Globe->ScreenToClient(MousePos), 1.0 / 1.1 );
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::GlobeMouseWheelUp(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  Globe->Projector->AdjustScaleFactor( Globe->ScreenToClient(MousePos), 1.1 );
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::Timer1Timer(TObject *Sender)
{
  Extended alpha = (double)FStepCounter / POZ_STEPS;
  TGPOZ poz;

  poz.iRotX = FPozStart.iRotX + (int)(( (double)FPozDest.iRotX - (double)FPozStart.iRotX ) * alpha );
  poz.iRotY = FPozStart.iRotY + (int)(( (double)FPozDest.iRotY - (double)FPozStart.iRotY ) * alpha );
  poz.iRotZ = FPozStart.iRotZ + (int)(( (double)FPozDest.iRotZ - (double)FPozStart.iRotZ ) * alpha );

  poz.CenterXY.x = FPozStart.CenterXY.x + (int)(( (double)FPozDest.CenterXY.x - (double)FPozStart.CenterXY.x ) * alpha );
  poz.CenterXY.y = FPozStart.CenterXY.y + (int)(( (double)FPozDest.CenterXY.y - (double)FPozStart.CenterXY.y ) * alpha );

  poz.eSF = FPozStart.eSF + ( FZoomInSF - FPozStart.eSF ) * alpha * alpha * alpha * alpha * alpha;

  poz.iWidth = FPozDest.iWidth;
  poz.iHeight = FPozDest.iHeight;

  Globe->SetPOZ(poz);

  ++FStepCounter;
  Timer1->Enabled = FStepCounter <= POZ_STEPS;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::StartEasing(int iLongX, int iLatY )
{
  Globe->GetPOZ(FPozStart);  // default starting values
  Globe->GetPOZ(FPozDest);  // Get destination values

  FPozStart.eSF = FZoomOutSF;
  FPozDest.eSF = FZoomInSF;

  FPozDest.CenterXY = Point( Globe->ClientWidth / 2, Globe->ClientHeight / 2 );
  FPozDest.iRotX = iLatY;
  FPozDest.iRotY = iLongX;

  FStepCounter = 0;
  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::GlobeMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  Timer1->Enabled = false;
}
//---------------------------------------------------------------------------

