//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "GSysUtils.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "GClasses"
#pragma link "Globe5"
#pragma link "GSpatialDatabase"
#pragma link "GMapObjects"
#pragma link "GMapObjectEditors"
#pragma link "GSysUtils"
#pragma resource "*.dfm"
TForm1 *Form1;

template<typename T>
T cast_interface( const TObject *obj )
{
  T result;

  if ( Supports( obj, __uuidof(T), &result ) == false )
    throw EIntfCastError( "Interface not supported" );

  return result;
};

 //---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
  // start the spatial database and zoom the globe to it's extents
  sdb->Active = true;
  Globe->Projector->ScaleFactor = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bAddImageClick(TObject *Sender)
{
  // make sure the image is present->
  if ( FileExists( "..\\MapData\\Earth2.bmp" ) == false )
  {
    ShowMessage( "..\\MapData\\Earth2.bmp is missing" );
    return;
  }

  bAddImage->Enabled = false;
  bGeoReference->Enabled = true;
  bEditImage->Enabled = true;


  // Create a Raster object for the image
  rasterObject = new TGMapRaster;
  rasterObject->ImageFilename = "..\\MapData\\Earth2.bmp";

  // Give the image an initial size and position on the globe
  rasterObject->Morph->ScaleX = GU_MINUTE * 5;
  rasterObject->Morph->ScaleY = GU_MINUTE * 5;
  rasterObject->Morph->OffsetX = -GU_10_DEGREE * 3;
  rasterObject->Morph->OffsetY = GU_10_DEGREE;

  // create a temporary layer and add the object
  imageLayer = new TGLayer(Globe);
  imageLayer->ObjectSource->Add( cast_interface<_di_IGMapPoint>(rasterObject));

  // add the layer to the globe
  Globe->GlobeLayer->LayerStore->Add(imageLayer);

}
//---------------------------------------------------------------------------
void __fastcall TForm1::bEditImageClick(TObject *Sender)
{
  if ( bEditImage->Down )
  {
    // create the editor and assign the object to it
    rasterEditor = new TGMapRasterEditor(Globe);
    rasterEditor->EditObject(cast_interface<_di_IGMapPoint>(rasterObject));
  }
  else
  {
    // This will save the changes to the object by default
    // call rasterEditor->CancelEdit to undo the changes
    delete rasterEditor;
  }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::bGeoReferenceClick(TObject *Sender)
{
  // remove any existing warping
  rasterObject->Morph->Clear();
  rasterObject->Morph->Add(); // add a control pair

  TGControlPair cp = rasterObject->Morph->ControlPairs[0];

  // Set the coordinates of the image line
  cp.ImageStart = Point(100,100);
  cp.ImageEnd = Point( 600, 300 );

  // Reference those coordinates to the globe
  cp.WorldStart = DDToPointLL(5, 5);
  cp.WorldEnd = DDToPointLL(15, 20);

  Globe->RedrawLayers();

}
//---------------------------------------------------------------------------
