//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include <assert.h>
#include "GWindBarbPresenter.hpp"
#include "GCSVReader.hpp"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "GClasses"
#pragma link "Globe5"
#pragma link "GSpatialDatabase"
#pragma link "GCSVReader"  // Must include GCSVReader so that the Winds.csv fiel can be loaded

#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
  TGLayer *aLayer;
  TGLayer *WindLayer;

  GSpatialDatabase->Active = true;

  // find the TGWorld data layer
  aLayer = Globe->GlobeLayer->LayerStore->LayerByName( "TGWorld" );
  assert( aLayer != NULL );

  WindLayer = Globe->GlobeLayer->LayerStore->LayerByName( "Wind Layer" );
  if ( WindLayer == NULL )
  {
    WindLayer = new TGLayer(Globe);
    WindLayer->Name = "Wind Layer";
    Globe->GlobeLayer->LayerStore->Add(WindLayer);
  }

  // Create a new poly presenter and add it to the layer
  PolyPresenter = new TGPolyPresenter(Globe);
  PolyPresenter->PresenterID = 1;
  aLayer->Presenters->Add(PolyPresenter);

  // Find the TGCities layer
  aLayer = Globe->GlobeLayer->LayerStore->LayerByName("TGCities" );
  assert( aLayer != NULL );

  // Create a new point presenter and add it to the layer
  PointPresenter = new TGPointPresenter(Globe);
  PointPresenter->PointBrush->Color = clRed;
  PointPresenter->PresenterID = 1;
  PointPresenter->TitleAlignment = taTopRight;
  aLayer->Presenters->Add(PointPresenter);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bBrushColorClick(TObject *Sender)
{
  ColorDialog1->Color = PolyPresenter->PolyBrush->Color;

  if ( ColorDialog1->Execute())
  {
    PolyPresenter->PolyBrush->Color = ColorDialog1->Color;
    Globe->RedrawLayers();
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bPenColorClick(TObject *Sender)
{
  ColorDialog1->Color = PolyPresenter->PolyBrush->Color;

  if ( ColorDialog1->Execute() )
  {
    PolyPresenter->PolyPen->Color = ColorDialog1->Color;
    Globe->RedrawLayers();
  }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::rbTCClick(TObject *Sender)
{
  UpdateTitleAlignment();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::UpdateTitleAlignment()
{
  if ( rbTL->Checked )
   PointPresenter->TitleAlignment = taTopLeft;
  if ( rbTC->Checked )
   PointPresenter->TitleAlignment = taTop;
  if ( rbTR->Checked )
   PointPresenter->TitleAlignment = taTopRight;
  if ( rbCL->Checked )
   PointPresenter->TitleAlignment = taLeft;
  if ( rbCC->Checked )
   PointPresenter->TitleAlignment = Gclasses::taCenter;
  if ( rbCR->Checked )
   PointPresenter->TitleAlignment = taRight;
  if ( rbBL->Checked )
   PointPresenter->TitleAlignment = taBottomLeft;
  if ( rbBC->Checked )
   PointPresenter->TitleAlignment = taBottom;
  if ( rbBR->Checked )
   PointPresenter->TitleAlignment = taBottomRight;

  Globe->RedrawLayers();
}
void __fastcall TForm1::seOffsetChange(TObject *Sender)
{
  PointPresenter->TitleOffset = StrToIntDef( seOffset->Text, 0 );
  Globe->RedrawLayers();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::cbPointTypeChange(TObject *Sender)
{
  PointPresenter->PointType = TGPointPresenterType( cbPointType->ItemIndex + 3 );
  Globe->RedrawLayers();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::sePointSizeChange(TObject *Sender)
{
  PointPresenter->PointSize = StrToIntDef( sePointSize->Text, 10 );
  Globe->RedrawLayers();
  
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bPointBrushColorClick(TObject *Sender)
{
  ColorDialog1->Color = PointPresenter->PointBrush->Color;

  if ( ColorDialog1->Execute() )
  {
    PointPresenter->PointBrush->Color = ColorDialog1->Color;
    Globe->RedrawLayers();
  }
 
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bPointPenColorClick(TObject *Sender)
{
  ColorDialog1->Color = PointPresenter->PointPen->Color;

  if ( ColorDialog1->Execute() )
  {
    PointPresenter->PointPen->Color = ColorDialog1->Color;
    Globe->RedrawLayers();
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bFontColorClick(TObject *Sender)
{
  ColorDialog1->Color = PointPresenter->TitleFont->Color;

  if ( ColorDialog1->Execute() )
  {
    PointPresenter->TitleFont->Color = ColorDialog1->Color;
    Globe->RedrawLayers();
  }
 
}
//---------------------------------------------------------------------------

void __fastcall TForm1::cbHaloClick(TObject *Sender)
{
  TGFontStyles fontStyle = PointPresenter->TitleFont->Style;

  if ( cbHalo->Checked )
    fontStyle << gfsHalo;
  else
    fontStyle >> gfsHalo;

  PointPresenter->TitleFont->Style = fontStyle;

  Globe->RedrawLayers();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::cbOpaqueClick(TObject *Sender)
{
  TGFontStyles fontStyle = PointPresenter->TitleFont->Style;

  if ( cbOpaque->Checked )
    fontStyle << gfsOpaque;
  else
    fontStyle >>gfsOpaque;

  PointPresenter->TitleFont->Style = fontStyle;

  Globe->RedrawLayers();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::bBackColorClick(TObject *Sender)
{
  ColorDialog1->Color = PointPresenter->TitleFont->BkColor;

  if ( ColorDialog1->Execute() )
  {
    PointPresenter->TitleFont->BkColor = ColorDialog1->Color;
    Globe->RedrawLayers();
  }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::bAddWindsClick(TObject *Sender)
{
  TGLayer *WindLayer = Globe->GlobeLayer->LayerStore->LayerByName( "Wind Layer" );

  if ( WindLayer != NULL )
  {
    WindLayer->Clear();

    TGWindBarbPresenter *WindPresenter = new TGWindBarbPresenter(Globe);
    WindPresenter->BarbColor = clWhite;
    WindPresenter->BarbLength = 20;
    WindPresenter->BarbUnit = guNauticalMile;
    WindPresenter->DirectionColumn = 8;
    WindPresenter->StrengthColumn = 7;
    WindPresenter->PresenterID = 1;

    WindLayer->Presenters->Add(WindPresenter);

    TGDataReader *dataReader = CreateFileDataReader(Globe, "Winds.CSV" );
    WindLayer->ObjectSource->ImportData( dataReader );
    delete dataReader;
  }
}
//---------------------------------------------------------------------------



