{------------------------------------------------------------------------------
  City Distance demo program

  Demonstration application to calculate the distance between the selected
  cities on the globe.

  Uses Easing to locate the cities when clicked in the city list

  Thanks to Chris Williams for updating this app from TGlobe 3 to TGlobe 4. 
------------------------------------------------------------------------------}
unit Umain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, Buttons, Menus,
  Globe5Utils, GSysUtils, GClasses, Globe5, GSpatialDatabase;

const
  POZ_STEPS = 16;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    pnlHint: TPanel;
    Panel3: TPanel;
    Splitter1: TSplitter;
    lbxCities: TListBox;
    edtFilter: TEdit;
    grdWayPoints: TStringGrid;
    Panel4: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Label1: TLabel;
    btnWorld: TSpeedButton;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    About1: TMenuItem;
    Splitter2: TSplitter;
    btnZoomIn: TBitBtn;
    btnZoomOut: TBitBtn;
    PopupMenu1: TPopupMenu;
    DeleteCity1: TMenuItem;
    N1: TMenuItem;
    AddCity1: TMenuItem;
    ModifyCity1: TMenuItem;
    LocateCity1: TMenuItem;
    Globe: TGlobe5;
    sdb: TGSpatialDatabase;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure edtFilterChange(Sender: TObject);
    procedure grdWayPointsDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure grdWayPointsDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure GlobeRender(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure lbxCitiesDblClick(Sender: TObject);
    procedure grdWayPointsDblClick(Sender: TObject);
    procedure btnWorldClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure btnZoomInClick(Sender: TObject);
    procedure btnZoomOutClick(Sender: TObject);
    procedure DeleteCity1Click(Sender: TObject);
    procedure LocateCity1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ModifyCity1Click(Sender: TObject);
    procedure AddCity1Click(Sender: TObject);
    procedure GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure Timer1Timer(Sender: TObject);
    procedure GlobeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    ctyLayer : TGLayer;
    FPozStart, FPozDest : TGPOZ;
    FStepCounter : integer;
    FZoomInSF, FZoomOutSF : Extended;
    procedure StartEasing( iLongX, iLatY : integer );
  public
    { Public declarations }
    procedure AddWayPoint( objPK : TGPrimaryKey );
    procedure BuildWayPointList;
    procedure ShowHint( Sender : TObject );
  end;

var
  frmMain: TfrmMain;
  WayPoints : TInterfaceList;

implementation

uses Uabout, Ucity;

{$R *.DFM}

{----------------------------------------------------------------------------}
procedure TfrmMain.ShowHint( Sender : TObject );
begin
  pnlHint.Caption := ' ' + GetLongHint(Application.Hint);
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  sdb.Active := true;
  WayPoints := TInterfaceList.Create;
  Application.OnHint := ShowHint;
  btnWorldClick( Sender );

  with grdWayPoints do
  begin
    Cells[0,0] := 'City';
    Cells[1,0] := 'Statute Miles';
    Cells[2,0] := 'Nautical Miles';
    Cells[3,0] := 'Kilometers';
    Cells[0, 1] := '     Total Distance';
  end;

  ctyLayer := Globe.GlobeLayer.LayerStore.LayerByName('TGCities');

  edtFilterChange( nil );
  Globe.Projector.ScaleFactor := 0;
  FZoomOutSF := Globe.Projector.ScaleFactor;
  FZoomInSF := Globe.Projector.ScaleFactor * 8;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.edtFilterChange(Sender: TObject);
var
  sTmp : string;
  obj : IGMapPoint;
begin
  lbxCities.Items.BeginUpdate;
  lbxCities.Items.Clear;
  sTmp := Trim( edtFilter.Text );

  if ctyLayer <> nil then
  begin
    obj := ctyLayer.ObjectSource.FirstObject;
    while obj <> nil do
    begin
      if ( sTmp = '' ) or (CompareText(sTmp, Copy(obj.Attribute[0], 1, Length( sTmp ))) = 0 ) then
        lbxCities.Items.AddObject(obj.Attribute[0],Pointer(obj.PrimaryKey));

      obj := ctyLayer.ObjectSource.NextObject( obj );
    end;
  end;

  lbxCities.Items.EndUpdate;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.lbxCitiesDblClick(Sender: TObject);
begin
  with lbxCities do
    AddWayPoint( TGPrimaryKey( Items.Objects[ItemIndex] ));
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.grdWayPointsDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := True;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.BuildWayPointList;
var
  idx, iTmp : integer;
  eM, eNM, eK : Extended;
  ptFrom, ptTo : TGPointLL;
begin
  with grdWayPoints do
  begin
    RowCount := WayPoints.Count + 2;
    for idx := 0 to WayPoints.Count - 1 do
      Cells[0, idx + 1] := IGMapPoint( WayPoints[idx] ).Attribute[0];

    Cells[0, RowCount - 1] := '   Total Distance';

    eM := 0;
    eNM := 0;
    eK := 0;
    if WayPoints.Count > 0 then
    begin
      ptFrom := IGMapPoint( WayPoints[0] ).Centroid;
      for idx := 0 to WayPoints.Count - 1 do
      begin
        ptTo := IGMapPoint( WayPoints[idx] ).Centroid;
        iTmp := DistanceLLToLL( ptFrom, ptTo, WGS84 );

        eM := eM + GUTo( iTmp, guMile );
        Cells[1, idx + 1] := FloatToStr( GUTo( iTmp, guMile ));

        eNM := eNM + GUTo( iTmp, guNauticalMile );
        Cells[2, idx + 1] := FloatToStr( GUTo( iTmp, guNauticalMile ));

        eK := eK + GUTo( iTmp, guKilometer );
        Cells[3, idx + 1] := FloatToStr( GUTo( iTmp, guKilometer ));

        ptFrom := ptTo;
      end;

    end;
    Cells[1, RowCount - 1] := FloatToStr( eM );
    Cells[2, RowCount - 1] := FloatToStr( eNM );
    Cells[3, RowCount - 1] := FloatToStr( eK );
  end;
end;


{----------------------------------------------------------------------------}
procedure TfrmMain.AddWayPoint( objPK : TGPrimaryKey );
begin
  if ctyLayer <> nil then
  begin
    WayPoints.Add( ctyLayer.ObjectSource.ByPrimaryKey( objPK ));
    BuildWayPointList;
    Globe.RedrawLayers;
  end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.grdWayPointsDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  with lbxCities do
    AddWayPoint( TGPrimaryKey( Items.Objects[ItemIndex] ));
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.GlobeRender(Sender: TObject);
var
  idx : integer;
  ptFrom, ptTo : TGPointLL;
begin
  with Globe.Renderer.Pen do
  begin
    Width := 2;
    Color := clRed;
    Style := gpsSolid;
  end;

  if WayPoints.Count > 1 then
  begin
    ptFrom := IGMapPoint( WayPoints[0] ).Centroid;
    for idx := 1 to WayPoints.Count - 1 do
    begin
      ptTo := IGMapPoint( WayPoints[idx] ).Centroid;
      Globe.RenderGreatCircleLine( ptFrom, ptTo, gcShortest );
      ptFrom := ptTo;
    end;
  end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.SpeedButton1Click(Sender: TObject);
begin
  Globe.Projector.ProjectionClass := 'TGSphericalPrj';
  btnWorldClick( Sender );
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.SpeedButton2Click(Sender: TObject);
begin
  Globe.Projector.ProjectionClass := 'TGMercatorPrj';
  btnWorldClick( Sender );
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.btnWorldClick(Sender: TObject);
begin
  Globe.Projector.ScaleFactor := 0;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.grdWayPointsDblClick(Sender: TObject);
begin
  with grdWayPoints do
    if ( Selection.Top > 0 ) and ( Selection.Top < RowCount - 1 ) then
    begin
      WayPoints.Delete( Selection.Top - 1 );
      BuildWayPointList;
      Globe.RedrawLayers;
    end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.About1Click(Sender: TObject);
begin
  frmAbout.ShowModal;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.btnZoomInClick(Sender: TObject);
begin
  with Globe.Projector do
    ScaleFactor := ScaleFactor * 2;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.btnZoomOutClick(Sender: TObject);
begin
  with Globe.Projector do
    ScaleFactor := ScaleFactor / 2;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ssCtrl in Shift then
  begin
    case Key of
      VK_ADD:
        btnZoomInClick( nil );
      VK_SUBTRACT:
        btnZoomOutClick( nil );
      else
        Exit;
    end;
    Key := 0;
  end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.PopupMenu1Popup(Sender: TObject);
var
  CursorPos : TPoint;
begin
  GetCursorPos(CursorPos);
  with lbxCities do
  begin
    ItemIndex := ItemAtPos(ScreenToClient(CursorPos), True);
    if ItemIndex = -1 then
      Abort;
  end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.DeleteCity1Click(Sender: TObject);
var
  idx : integer;
begin
  with lbxCities do
  begin
    for idx := 0 to WayPoints.Count - 1 do
      if IGMapPoint( WayPoints[idx] ).PrimaryKey = TGPrimaryKey(Items.Objects[ItemIndex]) then
      begin
        WayPoints.Delete( idx );
        BuildWayPointList;
        break;
      end;
    IGMapPoint( ctyLayer.ObjectSource.ByPrimaryKey( TGPrimaryKey(Items.Objects[ItemIndex] ))).Delete;
    Items.Delete( ItemIndex );
    Globe.RedrawLayers;
  end;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.LocateCity1Click(Sender: TObject);
begin
  if ctyLayer <> nil then
    with lbxCities do
      with IGMapPoint( ctyLayer.ObjectSource.ByPrimaryKey( TGPrimaryKey(Items.Objects[ItemIndex] ))).Centroid do
        StartEasing( iLongX, iLatY );
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.ModifyCity1Click(Sender: TObject);
begin
  with lbxCities do
    frmCityObject.ModifyCity(IGMapPoint( ctyLayer.ObjectSource.ByPrimaryKey( TGPrimaryKey(Items.Objects[ItemIndex] ))));
  edtFilterChange(nil);
  Globe.RedrawLayers;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.AddCity1Click(Sender: TObject);
begin
  frmCityObject.NewCity( ctyLayer );
  edtFilterChange(nil);
  Globe.RedrawLayers;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  TGlobe5(Sender).Projector.AdjustScaleFactor( TGlobe5(Sender).ScreenToClient(MousePos), 1.0 / 1.1 );
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  TGlobe5(Sender).Projector.AdjustScaleFactor( TGlobe5(Sender).ScreenToClient(MousePos), 1.1 );
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.Timer1Timer(Sender: TObject);
var
  alpha : Extended;
  poz : TGPOZ;
begin
  alpha := FStepCounter / POZ_STEPS;
  poz.iRotX := FPozStart.iRotX + Round(( FPozDest.iRotX - FPozStart.iRotX ) * alpha );
  poz.iRotY := FPozStart.iRotY + Round(( FPozDest.iRotY - FPozStart.iRotY ) * alpha );
  poz.iRotZ := FPozStart.iRotZ + Round(( FPozDest.iRotZ - FPozStart.iRotZ ) * alpha );

  poz.CenterXY.X := FPozStart.CenterXY.X + Round(( FPozDest.CenterXY.X - FPozStart.CenterXY.X ) * alpha );
  poz.CenterXY.Y := FPozStart.CenterXY.Y + Round(( FPozDest.CenterXY.Y - FPozStart.CenterXY.Y ) * alpha );

  poz.eSF := FPozStart.eSF + ( FZoomInSF - FPozStart.eSF ) * alpha * alpha * alpha * alpha * alpha;

  poz.iWidth := FPozDest.iWidth;
  poz.iHeight := FPozDest.iHeight;

  Globe.SetPOZ(poz);

  Inc( FStepCounter );
  Timer1.Enabled := FStepCounter <= POZ_STEPS;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.StartEasing( iLongX, iLatY : integer );
begin
  Globe.GetPOZ(FPozStart);  // default starting values
  Globe.GetPOZ(FPozDest);  // Get destination values

  FPozStart.eSF := FZoomOutSF;

  FPozDest.CenterXY := Point( Globe.ClientWidth div 2, Globe.ClientHeight div 2 );
  FPozDest.iRotX := iLatY;
  FPozDest.iRotY := iLongX;

  FStepCounter := 0;
  Timer1.Enabled := true;
end;

{----------------------------------------------------------------------------}
procedure TfrmMain.GlobeMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Timer1.Enabled := false;
end;

end.
