{------------------------------------------------------------------------------
  Easing demo program

  Simple application to show smooth animation of the globe by easing from one
  location and zoom setting to another.

  Uses the GetPOZ and SetPOZ methods to record and set the globe state.
  POZ = Position Orientation and Zoom
------------------------------------------------------------------------------}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, GClasses, Globe5, GSpatialDatabase, GSysUtils,
  StdCtrls;

const
  POZ_STEPS = 20;

type
  TForm1 = class(TForm)
    sdb: TGSpatialDatabase;
    Globe2: TGlobe5;
    Panel1: TPanel;
    Globe1: TGlobe5;
    Timer1: TTimer;
    Label1: TLabel;
    Label2: TLabel;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Globe1Rotated(Sender: TObject);
    procedure Globe1Zoomed(Sender: TObject);
    procedure Globe1MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure Globe1MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    FPozStart, FPozDest : TGPOZ;
    FStepCounter : integer;
  public
    { Public declarations }
    procedure StartEasing;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  sdb.Active := true;
end;

procedure TForm1.StartEasing;
begin
  Globe2.GetPOZ(FPozStart);  // default starting values
  Globe1.GetPOZ(FPozDest);  // Get destination values

  // normalise for the different globe component sizes
  POZ_Normalise(FPozDest,Globe2.ClientWidth, Globe2.ClientHeight);

  FStepCounter := 0;
  Timer1.Enabled := true;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  alpha : Extended;
  poz : TGPOZ;
begin
  // Performs the easing from the starting POZ to the destination POZ
  alpha := FStepCounter / POZ_STEPS;

  // Adjust the Orientation of the Globe.
  poz.iRotX := FPozStart.iRotX + Round(( FPozDest.iRotX - FPozStart.iRotX ) * alpha );
  poz.iRotY := FPozStart.iRotY + Round(( FPozDest.iRotY - FPozStart.iRotY ) * alpha );
  poz.iRotZ := FPozStart.iRotZ + Round(( FPozDest.iRotZ - FPozStart.iRotZ ) * alpha );

  // Adjust the globe center
  poz.CenterXY.X := FPozStart.CenterXY.X + Round(( FPozDest.CenterXY.X - FPozStart.CenterXY.X ) * alpha );
  poz.CenterXY.Y := FPozStart.CenterXY.Y + Round(( FPozDest.CenterXY.Y - FPozStart.CenterXY.Y ) * alpha );

  // Adjust the globe zoom
  poz.eSF := FPozStart.eSF + ( FPozDest.eSF - FPozStart.eSF ) * alpha;
  poz.iWidth := FPozDest.iWidth;
  poz.iHeight := FPozDest.iHeight;

  Globe2.SetPOZ(poz);

  Inc( FStepCounter );
  Timer1.Enabled := FStepCounter <= POZ_STEPS;
end;

procedure TForm1.Globe1Rotated(Sender: TObject);
begin
  StartEasing;
end;

procedure TForm1.Globe1Zoomed(Sender: TObject);
begin
  StartEasing;
end;

procedure TForm1.Globe1MouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // zooms the globe about the current mouse location using the mouse wheel.
  TGlobe5(Sender).Projector.AdjustScaleFactor( TGlobe5(Sender).ScreenToClient(MousePos), 1.0 / 1.1 );
end;

procedure TForm1.Globe1MouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // zooms the globe about the current mouse location using the mouse wheel.
  TGlobe5(Sender).Projector.AdjustScaleFactor( TGlobe5(Sender).ScreenToClient(MousePos), 1.1 );
end;

end.
