{------------------------------------------------------------------------------
  GeoReferencing demo program

  Shows how to add an image to a Globe layer, then how to use the
  TGRasterEditor component to visually edit and manipulate the image.

  The editor requires the user to hold down the Ctrl key for most of the
  edit operations.
------------------------------------------------------------------------------}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  GSpatialDatabase, GClasses, Globe5, GMapObjects, GSysUtils, GMapObjectEditors,
  Buttons;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Globe: TGlobe5;
    sdb: TGSpatialDatabase;
    bAddImage: TButton;
    SpeedButton1: TSpeedButton;
    bEditImage: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    bGeoReference: TSpeedButton;
    procedure bAddImageClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bEditImageClick(Sender: TObject);
    procedure bGeoReferenceClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    imageLayer : TGLayer;
    rasterObject : TGMapRaster;
    rasterEditor : TGMapRasterEditor;
  end;

var
  Form1: TForm1;

implementation

uses GMorphing;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  // start the spatial database and zoom the globe to it's extents
  sdb.Active := true;
  Globe.Projector.ScaleFactor := 0;
end;

procedure TForm1.bAddImageClick(Sender: TObject);
begin
  // make sure the image is present.
  if not FileExists( '..\MapData\Earth2.bmp' ) then
  begin
    ShowMessage( '..\MapData\Earth2.bmp is missing' );
    Exit;
  end;

  bAddImage.Enabled := false;
  bGeoReference.Enabled := true;
  bEditImage.Enabled := true;


  // Create a Raster object for the image
  rasterObject := TGMapRaster.Create;
  rasterObject.ImageFilename := '..\MapData\Earth2.bmp';

  // Give the image an initial size and position on the globe.
  rasterObject.Morph.ScaleX := GU_MINUTE;
  rasterObject.Morph.ScaleY := GU_MINUTE;
  rasterObject.Morph.OffsetX := -GU_10_DEGREE * 3;
  rasterObject.Morph.OffsetY := GU_10_DEGREE;

  // create a temporary layer and add the object
  imageLayer := TGLayer.Create(Globe);
  imageLayer.ObjectSource.Add(rasterObject);

  // add the layer to the globe.
  Globe.GlobeLayer.LayerStore.Add(imageLayer);
end;

procedure TForm1.bEditImageClick(Sender: TObject);
begin
  if bEditImage.Down then
  begin
    // create the editor and assign the object to it.
    rasterEditor := TGMapRasterEditor.Create(Globe);
    rasterEditor.EditObject(rasterObject as IGMapPoint);
  end
  else
  begin
    // This will save the changes to the object by default.
    // call rasterEditor.CancelEdit to undo the changes
    rasterEditor.Free;
  end;
end;

procedure TForm1.bGeoReferenceClick(Sender: TObject);
begin
  // remove any existing warping.
  rasterObject.Morph.Clear;
  rasterObject.Morph.Add; // add a control pair

  with rasterObject.Morph.ControlPairs[0] do
  begin
    // Set the coordinates of the image line
    ImageStart := Point(100,100);
    ImageEnd := Point( 600, 300 );

    // Reference those coordinates to the globe
    WorldStart := DDToPointLL(5, 5);
    WorldEnd := DDToPointLL(15, 20);
  end;

  Globe.RedrawLayers;
end;

end.
