{------------------------------------------------------------------------------
  Data Import demo program

  Demonstrates loading data into a spatial database.
------------------------------------------------------------------------------}
unit FormImport;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, GSpatialDatabase, GClasses, Globe5, GSysUtils,
  GDataReaders;

type
  TForm1 = class(TForm)
    Globe: TGlobe5;
    sdb: TGSpatialDatabase;
    bImport: TButton;
    OpenDialog1: TOpenDialog;
    lbLayers: TListBox;
    Panel1: TPanel;
    bClear: TButton;
    procedure FormCreate(Sender: TObject);
    procedure bImportClick(Sender: TObject);
    procedure bClearClick(Sender: TObject);
    procedure sdbChanged(Sender: TObject);
    procedure GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    procedure LoadDataLayer(const filename: String);

    procedure UpdateLayerList;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

// The following data reader units must be included so that they are
// registered with the Delphi Class registration mechanism. This mechanism
// is used by the CreateFileDataReader() function to create the appropriate
// data reader for the type of file to import.

Uses GLR5Reader, GSHPReader, GMIFReader, GLYRReader;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  // Create or open a new Spatial database with the same name as the app.
  sdb.DataSourceName := ChangeFileExt( Application.ExeName, '.sdb' );
  sdb.Active := true;
end;

procedure TForm1.bImportClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    LoadDataLayer(OpenDialog1.FileName);
end;

procedure TForm1.LoadDataLayer( const filename : String );
var
  layerReader : TGFileDataReader;
  layerID : TGUID;
begin
  // Creates a data reader for the file. See Help file for more infomation on
  // this function. ( CreateFileDataReader() is in the GDataReaders unit )
  layerReader := CreateFileDataReader( Globe, filename );

  if layerReader <> nil then
  try
    Screen.Cursor := crHourGlass;

    try
      // Imports the layer into the SDB and returns a unique ID for the layer.
      layerID := sdb.ImportLayer( layerReader );

      if IsEqualGUID( layerID, NULL_GUID ) then
        ShowMessage( 'Failed to load file: ' + filename );
    except
      on E:EGException do
        ShowMessage( E.Message );
    end;
  finally
    FreeAndNil( layerReader );
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.bClearClick(Sender: TObject);
begin
  sdb.Clear;
end;

procedure TForm1.sdbChanged(Sender: TObject);
begin
  UpdateLayerList;
end;

procedure TForm1.UpdateLayerList;
var
  idx : integer;
begin
  lbLayers.Clear;

  // Display a list of layers held in the spatial database
  for idx := 0 to sdb.SDBLayers.Count - 1 do
    lbLayers.Items.Add(sdb.SDBLayers[idx].Name );
end;

procedure TForm1.GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // Zooms the globe centered about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.0 / 1.1 );
end;

procedure TForm1.GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // Zooms the globe centered about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.1 );
end;

end.
