unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GSpatialDatabase, GClasses, Globe5, ComCtrls, GLayerTreeView,
  ImgList, Menus, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Globe: TGlobe5;
    sdb: TGSpatialDatabase;
    ImageList1: TImageList;
    pumStart: TPopupMenu;
    pumPresenter: TPopupMenu;
    PumLayer: TPopupMenu;
    pumLayerStore: TPopupMenu;
    pumPresenterStore: TPopupMenu;
    PresenterStoremenu1: TMenuItem;
    Presentermenu1: TMenuItem;
    LayerStoremenu1: TMenuItem;
    Layermenu1: TMenuItem;
    pumObjectSource: TPopupMenu;
    MenuItem1: TMenuItem;
    Panel1: TPanel;
    GLayerTreeView: TGLayerTreeView;
    Panel2: TPanel;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure pumStartPopup(Sender: TObject);
    procedure GLayerTreeViewEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure GLayerTreeViewEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure GLayerTreeViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure GLayerTreeViewDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure GLayerTreeViewDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
//------------------------------------------------------------------------------
// Loading a saving of an environment file for the spatial database
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  sdb.Active := true;
  Globe.LoadEnvironmentFromFile( ChangeFileExt( Application.ExeName, '.env' ));
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Globe.SaveEnvironmentToFile(ChangeFileExt( Application.ExeName, '.env' ));
end;


//------------------------------------------------------------------------------
// Controlling the popup menu displayed for the layer tree
//------------------------------------------------------------------------------
procedure TForm1.pumStartPopup(Sender: TObject);
begin
  // controls which popup menu to show depending on the node selected
  if GLayerTreeView.Selected <> nil then
    with pumStart.PopupPoint do
    begin
      if TObject( GLayerTreeView.Selected.Data ) is TGPresenterStore then
        pumPresenterStore.Popup(X, Y);

      if TObject( GLayerTreeView.Selected.Data ) is TGLayerStore then
        pumLayerStore.Popup(X, Y);

      if TObject( GLayerTreeView.Selected.Data ) is TGLayer then
        pumLayer.Popup(X, Y);

      if TObject( GLayerTreeView.Selected.Data ) is TGPresenter then
        pumPresenter.Popup(X, Y);

      if TObject( GLayerTreeView.Selected.Data ) is TGObjectSource then
        pumObjectSource.Popup(X, Y);
    end;
  Abort;  // we don't want to see this popup menu at all.
end;

//------------------------------------------------------------------------------
// Editing of the text in the layer tree
//------------------------------------------------------------------------------
procedure TForm1.GLayerTreeViewEditing(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  AllowEdit := ( TObject(Node.Data) is TGLayer ) or ( TObject(Node.Data) is TGPresenter );
end;

procedure TForm1.GLayerTreeViewEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
begin
  if TObject(Node.Data) is TGLayer then
    TGLayer(Node.Data).Name := S;

  if TObject(Node.Data) is TGPresenter then
    TGPresenter(Node.Data).Name := S;
end;

procedure TForm1.GLayerTreeViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ( Key = VK_F2 ) and ( GLayerTreeView.Selected <> nil ) then
    GLayerTreeView.Selected.EditText;
end;

//------------------------------------------------------------------------------
// drag and drop of layers within the layer tree
//------------------------------------------------------------------------------
procedure TForm1.GLayerTreeViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  layerStore : TGLayerStore;
begin
  Accept := false;
  if Source = GLayerTreeView then
  begin
    layerStore := GLayerTreeView.NodeToLayerStore(GLayerTreeView.GetNodeAt( X, Y ));
    Accept := ( layerStore <> nil ) and ( TObject( GLayerTreeView.Selected.Data ) is TGLayer );
  end;
end;

procedure TForm1.GLayerTreeViewDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  layerStore : TGLayerStore;
  node : TTreeNode;
  idx, index : integer;

  function IsValidLayer( aLayer : TGLayer ) : Boolean;
  var
    tmpLayer : TGLayer;
  begin
    // Make sure we are not dropping onto ourself
    Result := false;
    if layerStore = nil then
      Exit;
    tmpLayer := layerStore.ParentLayer;

    while tmplayer <> nil do
    begin
      if tmpLayer = aLayer then
        Exit;
      tmpLayer := tmpLayer.ParentLayer;
    end;
    Result := true;
  end;

begin
  if Source = GLayerTreeView then
  begin
    Globe.BeginUpdate;

    node := GLayerTreeView.GetNodeAt( X, Y );
    layerStore := GLayerTreeView.NodeToLayerStore( node );

    // add the layers to the layerstore
    if layerStore <> nil then
      for idx := 0 to GLayerTreeView.SelectionCount - 1 do
        if TObject( GLayerTreeView.Selections[idx].Data ) is TGLayer then
          if IsValidLayer( TGLayer( GLayerTreeView.Selections[idx].Data )) then
          begin
            index := layerStore.Add( TGLayer( GLayerTreeView.Selections[idx].Data ));

            if TObject( node.Data ) is TGlayer then
              layerStore.Move(index, TGLayer( node.Data ).Index);
          end;

    // if Source parent is ancestor of target
    if node.HasAsParent(GLayerTreeView.Selected.Parent) then
      GLayerTreeView.RebuildNode( GLayerTreeView.Selected.Parent )
    else
      if GLayerTreeView.Selected.HasAsParent(node.Parent) then   // if target parent is ancestor of source
        GLayerTreeView.RebuildNode( node.Parent )
      else
      begin
        GLayerTreeView.RebuildNode( GLayerTreeView.Selected.Parent );
        GLayerTreeView.RebuildNode( node.Parent );
      end;
    Globe.EndUpdate;
  end;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  if CheckBox1.Checked then
    GLayerTreeView.Options := GLayerTreeView.Options + [toHideLayerPresenters]
  else
    GLayerTreeView.Options := GLayerTreeView.Options - [toHideLayerPresenters];
end;

procedure TForm1.CheckBox2Click(Sender: TObject);
begin
  if CheckBox2.Checked then
    GLayerTreeView.Options := GLayerTreeView.Options + [toHideObjectSourcePresenters]
  else
    GLayerTreeView.Options := GLayerTreeView.Options - [toHideObjectSourcePresenters];
end;

procedure TForm1.CheckBox3Click(Sender: TObject);
begin
  if CheckBox3.Checked then
    GLayerTreeView.Options := GLayerTreeView.Options + [toHideObjectSource]
  else
    GLayerTreeView.Options := GLayerTreeView.Options - [toHideObjectSource];
end;

end.
