{------------------------------------------------------------------------------
  Object Creation demo program

  Shows the process of adding point and poly objects to the globe.

  The polyline object is displayed using a TGRoadPresenter, try changing the
  properties of the Presenter to get different effects.

------------------------------------------------------------------------------}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons,
  Globe5, GClasses, GSpatialDatabase, GMapObjects, GSysUtils, GPresenters,
  GRoadPresenter, StdCtrls;

type
  TForm1 = class(TForm)
    Globe: TGlobe5;
    GSpatialDatabase: TGSpatialDatabase;
    Panel1: TPanel;
    bAddPoint: TSpeedButton;
    bAddPolyline: TSpeedButton;
    bAddPolygon: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure GlobeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure bAddPolylineClick(Sender: TObject);
    procedure bAddPolygonClick(Sender: TObject);
    procedure bAddPointClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }
    procedure SavePoly;
  public
    { Public declarations }
    FPolyObject : IGMapPoly;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  aLayer : TGLayer;
  pointPresenter : TGPointPresenter;
  polygonPresenter : TGPolyPresenter;
  roadPresenter : TGRoadPresenter;
  LayerID : TGUID;
begin
  // Connect to the spatial database
  GSpatialDatabase.Active := true;

  // Creates a Permanent layer which is persisted in the Spatial Database
  aLayer := Globe.GlobeLayer.LayerStore.LayerByName('Temp Layer');
  if aLayer = nil then
  begin
    LayerID := GSpatialDatabase.CreateLayer('Temp Layer');
    aLayer := Globe.GlobeLayer.LayerStore.LayerByID(LayerID);
  end;

  // Create a Presenter for the point objects
  pointPresenter := TGPointPresenter.Create( Globe );
  pointPresenter.PresenterID := 1;
  pointPresenter.PointSize := 10;
  pointPresenter.PointType := ppTriangleUp;
  aLayer.Presenters.Add(pointPresenter);

  // Create a Road Presenter for the polyline objects
  roadPresenter := TGRoadPresenter.Create( Globe );
  roadPresenter.PresenterID := 2;
  roadPresenter.RoadPen.Color := clLime;
  roadPresenter.RoadPen.Width := 3;
  roadPresenter.RoadPen.WidthUnit := guPixel;
  roadPresenter.EdgePixels := 2;
  aLayer.Presenters.Add(roadPresenter);

  // Create a Presenter for the polygon objects
  polygonPresenter := TGPolyPresenter.Create( Globe );
  polygonPresenter.PresenterID := 3;
  polygonPresenter.PolyBrush.Color := clGreen;
  polygonPresenter.PolyPen.Color := clWhite;
  aLayer.Presenters.Add(polygonPresenter);
end;

procedure TForm1.GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // Zooms the globe about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.1 );
end;

procedure TForm1.GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // Zooms the globe about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.0 / 1.1 );
end;

procedure TForm1.GlobeMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  aLayer : TGLayer;
  obj : TGMapPoint;
  ptLL : TGPointLL;
begin
  // If adding points objects
  if bAddPoint.Down then
  begin
    // and the point is on the globe.
    if Globe.Projector.DeviceXYToLL(X,Y,ptLL) then
    begin
      aLayer := Globe.GlobeLayer.LayerStore.LayerByName('Temp Layer');
      Assert( aLayer <> nil, 'Temp Layer missing from Globe' );

      // Create a new map point object and set it's location
      obj := TGMapPoint.Create;
      obj.Centroid := ptLL;
      obj.PresenterID := 1;
      obj.Invalidate;

      // Add the object to the layer.
      aLayer.ObjectSource.Add(obj);
      Globe.RedrawLayers; // refresh the display
    end;
  end;

  // Create a Polygon or PolyLine
  if FPolyObject <> nil then
  begin
    // and the point is on the globe.
    if Globe.Projector.DeviceXYToLL(X,Y,ptLL) then
    begin
      // add the point to the first chain in the poly object
      FPolyObject.Chains[0].Add(ptLL);
      (FPolyObject as IGMapPoint).Invalidate; // make sure the object is updated.

      Globe.RedrawLayers; // refresh the display
    end;
  end;
end;

procedure TForm1.SavePoly;
begin
  if FPolyObject <> nil then
    (FPolyObject as IGMapPoint).Post;

  FPolyObject := nil;
end;

procedure TForm1.bAddPolylineClick(Sender: TObject);
var
  aLayer : TGLayer;
begin
  SavePoly;

  if bAddPolyline.Down then
  begin
    // find the temp layer
    aLayer := Globe.GlobeLayer.LayerStore.LayerByName('Temp Layer');
    Assert( aLayer <> nil, 'Temp Layer missing from Globe' );

    // create a new poly object
    FPolyObject := TGMapPoly.Create;
    FPolyObject.Closed := false;   // polylines are not closed.

    (FPolyObject as IGMapPoint).PresenterID := 2; // Use the Road Presenter

    aLayer.ObjectSource.Add(FPolyObject as IGMapPoint); // add the object to the layer
    (FPolyObject as IGMapPoint).Edit;
  end;
end;

procedure TForm1.bAddPolygonClick(Sender: TObject);
var
  aLayer : TGLayer;
begin
  SavePoly;

  if bAddPolygon.Down then
  begin
    // find the temp layer
    aLayer := Globe.GlobeLayer.LayerStore.LayerByName('Temp Layer');
    Assert( aLayer <> nil, 'Temp Layer missing from Globe' );

    // create a new poly object
    FPolyObject := TGMapPoly.Create;
    FPolyObject.Closed := true;   // polygons are closed.

    (FPolyObject as IGMapPoint).PresenterID := 3; // Use the polygon presenter

    aLayer.ObjectSource.Add(FPolyObject as IGMapPoint); // add the object to the layer

    (FPolyObject as IGMapPoint).Edit;
  end;
end;

procedure TForm1.bAddPointClick(Sender: TObject);
begin
  SavePoly;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  SavePoly;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
var
  aLayer : TGLayer;
begin
  SavePoly;

  aLayer := Globe.GlobeLayer.LayerStore.LayerByName('Temp Layer');
  if aLayer <> nil then
    aLayer.Clear;
end;

end.
