{------------------------------------------------------------------------------
  Presenter property demo program

  Shows the effects that changing some of the properties on a Presenter can
  have on the display of the map data.
------------------------------------------------------------------------------}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, GSpatialDatabase, GClasses, Globe5, GPresenters,
  GSysUtils, StdCtrls, Spin, GWindBarbPresenter, GMapObjects, Buttons,
  GDataReaders, GCSVReader;

  // Must include GCSVReader so that the Winds.csv file can be loaded

type
  TForm1 = class(TForm)
    Globe: TGlobe5;
    GSpatialDatabase: TGSpatialDatabase;
    Panel1: TPanel;
    ColorDialog1: TColorDialog;
    bBrushColor: TButton;
    bPointPenColor: TButton;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    rbTL: TRadioButton;
    rbTC: TRadioButton;
    rbTR: TRadioButton;
    rbCL: TRadioButton;
    rbCC: TRadioButton;
    rbCR: TRadioButton;
    rbBL: TRadioButton;
    rbBC: TRadioButton;
    rbBR: TRadioButton;
    Label2: TLabel;
    cbPointType: TComboBox;
    seOffset: TSpinEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    sePointSize: TSpinEdit;
    bPointBrushColor: TButton;
    bPenColor: TButton;
    bFontColor: TButton;
    cbHalo: TCheckBox;
    cbOpaque: TCheckBox;
    bBackColor: TButton;
    GroupBox2: TGroupBox;
    bAddWinds: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure bBrushColorClick(Sender: TObject);
    procedure bPenColorClick(Sender: TObject);
    procedure cbShowTitlesClick(Sender: TObject);
    procedure seOffsetChange(Sender: TObject);
    procedure cbPointTypeChange(Sender: TObject);
    procedure sePointSizeChange(Sender: TObject);
    procedure bPointBrushColorClick(Sender: TObject);
    procedure bPointPenColorClick(Sender: TObject);
    procedure bFontColorClick(Sender: TObject);
    procedure cbHaloClick(Sender: TObject);
    procedure cbOpaqueClick(Sender: TObject);
    procedure bBackColorClick(Sender: TObject);
    procedure bAddWindsClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    SatWindLayer : TGLayer;
    PolyPresenter : TGPolyPresenter;
    PointPresenter : TGPointPresenter;
    procedure UpdateTitleAlignment;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var
  aLayer, WindLayer : TGLayer;
begin
  GSpatialDatabase.Active := true;

  // find the TGWorld data layer
  aLayer := Globe.GlobeLayer.LayerStore.LayerByName('TGWorld' );
  Assert( aLayer <> nil, 'TGWorld layer missing from database' );

  WindLayer := Globe.GlobeLayer.LayerStore.LayerByName('Wind Layer');
  if WindLayer = nil then
  begin
    WindLayer := TGLayer.Create(Globe);
    WindLayer.Name := 'Wind Layer';
    Globe.GlobeLayer.LayerStore.Add(WindLayer);
  end;

  // Create a new poly presenter and add it to the layer
  PolyPresenter := TGPolyPresenter.Create(Globe);
  PolyPresenter.PresenterID := 1;
  aLayer.Presenters.Add(PolyPresenter);

  // Find the TGCities layer
  aLayer := Globe.GlobeLayer.LayerStore.LayerByName('TGCities' );
  Assert( aLayer <> nil, 'TGCities layer missing from database' );

  // Create a new point presenter and add it to the layer
  PointPresenter := TGPointPresenter.Create(Globe);
  PointPresenter.PointBrush.Color := clRed;
  PointPresenter.PresenterID := 1;
  PointPresenter.TitleAlignment := taTopRight;
  aLayer.Presenters.Add(PointPresenter);
end;

procedure TForm1.bBrushColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PolyPresenter.PolyBrush.Color;

  if ColorDialog1.Execute then
  begin
    PolyPresenter.PolyBrush.Color := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.bPenColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PolyPresenter.PolyPen.Color;

  if ColorDialog1.Execute then
  begin
    PolyPresenter.PolyPen.Color := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.cbShowTitlesClick(Sender: TObject);
begin
  UpdateTitleAlignment;
end;

procedure TForm1.UpdateTitleAlignment;
begin
  if rbTL.Checked then
   PointPresenter.TitleAlignment := taTopLeft;
  if rbTC.Checked then
   PointPresenter.TitleAlignment := taTop;
  if rbTR.Checked then
   PointPresenter.TitleAlignment := taTopRight;
  if rbCL.Checked then
   PointPresenter.TitleAlignment := taLeft;
  if rbCC.Checked then
   PointPresenter.TitleAlignment := taCenter;
  if rbCR.Checked then
   PointPresenter.TitleAlignment := taRight;
  if rbBL.Checked then
   PointPresenter.TitleAlignment := taBottomLeft;
  if rbBC.Checked then
   PointPresenter.TitleAlignment := taBottom;
  if rbBR.Checked then
   PointPresenter.TitleAlignment := taBottomRight;

  Globe.RedrawLayers;
end;

procedure TForm1.seOffsetChange(Sender: TObject);
begin
  PointPresenter.TitleOffset := seOffset.Value;
  Globe.RedrawLayers;
end;

procedure TForm1.cbPointTypeChange(Sender: TObject);
begin
  PointPresenter.PointType := TGPointPresenterType( cbPointType.ItemIndex + 3 );
  Globe.RedrawLayers;
end;

procedure TForm1.sePointSizeChange(Sender: TObject);
begin
  PointPresenter.PointSize := sePointSize.Value;
  Globe.RedrawLayers;
end;

procedure TForm1.bPointBrushColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PointPresenter.PointBrush.Color;

  if ColorDialog1.Execute then
  begin
    PointPresenter.PointBrush.Color := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.bPointPenColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PointPresenter.PointPen.Color;

  if ColorDialog1.Execute then
  begin
    PointPresenter.PointPen.Color := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.bFontColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PointPresenter.TitleFont.Color;

  if ColorDialog1.Execute then
  begin
    PointPresenter.TitleFont.Color := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.cbHaloClick(Sender: TObject);
var
  fontStyle : TGFontStyles;
begin
  fontStyle := PointPresenter.TitleFont.Style;

  if cbHalo.Checked then
    Include( fontStyle, gfsHalo )
  else
    Exclude( fontStyle, gfsHalo );

  PointPresenter.TitleFont.Style := fontStyle;

  Globe.RedrawLayers;
end;

procedure TForm1.cbOpaqueClick(Sender: TObject);
var
  fontStyle : TGFontStyles;
begin
  fontStyle := PointPresenter.TitleFont.Style;

  if cbOpaque.Checked then
    Include( fontStyle, gfsOpaque )
  else
    Exclude( fontStyle, gfsOpaque );

  PointPresenter.TitleFont.Style := fontStyle;

  Globe.RedrawLayers;
end;

procedure TForm1.bBackColorClick(Sender: TObject);
begin
  ColorDialog1.Color := PointPresenter.TitleFont.BkColor;

  if ColorDialog1.Execute then
  begin
    PointPresenter.TitleFont.BkColor := ColorDialog1.Color;
    Globe.RedrawLayers;
  end;
end;

procedure TForm1.bAddWindsClick(Sender: TObject);
var
 WindLayer : TGLayer;
 dataReader : TGDataReader;
 WindPresenter : TGWindBarbPresenter;
begin
  WindLayer := Globe.GlobeLayer.LayerStore.LayerByName('Wind Layer');
  if WindLayer <> nil then
  begin
    WindLayer.Clear;

    WindPresenter := TGWindBarbPresenter.Create(Globe);
    With WindPresenter do
    begin
      BarbColor := clWhite;
      BarbLength := 20;
      BarbUnit := guNauticalMile;
      DirectionColumn := 8;
      StrengthColumn := 7;
      PresenterID := 1;
    end;
    WindLayer.Presenters.Add(WindPresenter);

    dataReader := CreateFileDataReader(Globe, 'Winds.csv');
    WindLayer.ObjectSource.ImportData( dataReader );
    dataReader.Free;
  end;
end;

end.
