{------------------------------------------------------------------------------
	WorldClock demo program

 	Shows the sun shadow on the globe's surface.

------------------------------------------------------------------------------}
unit UMain;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, StdCtrls, Buttons,
  GClasses, GSysUtils, Globe5Utils, GSpatialDatabase, Globe5;

const
	TITLE = 'World Clock : ';

type
	TForm1 = class(TForm)
		Panel1: TPanel;
		cbxAuto: TCheckBox;
		Timer1: TTimer;
    sbDate: TScrollBar;
    sbTime: TScrollBar;
    Label1: TLabel;
    Label2: TLabel;
    SpeedButton1: TSpeedButton;
    SpeedButton3: TSpeedButton;
    Globe: TGlobe5;
    SpatialDatabase: TGSpatialDatabase;
		procedure GlobeRender(Sender: TObject);
		procedure FormResize(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure Timer1Timer(Sender: TObject);
		procedure sbDateChange(Sender: TObject);
    procedure GlobePaint(Sender: TObject);
    procedure GlobePaintGraticule(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
	private
		{ Private declarations }
	public
		{ Public declarations }
	end;

var
	Form1: TForm1;
	gDateTime, gFirstDayOfYear : TDateTime;


implementation

{$R *.DFM}

{----------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
var
	iYear,iTmp : word;
begin
  // calculate the first day of the current year
	DecodeDate( date, iYear, iTmp, iTmp );
	gFirstDayOfYear := EncodeDate( iYear, 1, 1 );
	Timer1Timer( Sender );

  // loads a bitmap image file if found otherwise shows the spatial database data
  if FileExists('..\MapData\Earth2.bmp') then
  	Globe.SurfaceTextureName := '..\MapData\Earth2.bmp'
  else
    SpatialDatabase.Active := true;
end;

{----------------------------------------------------------------------------}
procedure TForm1.FormResize(Sender: TObject);
begin
	Globe.Projector.ScaleFactor := 0;
end;

{----------------------------------------------------------------------------}
procedure TForm1.GlobeRender(Sender: TObject);
var
	idx : integer;
	ptSun : TGPointLL;
begin
  with Globe do
	begin
    // renders the suns shadow and returns the location of the sun over the earth
    ptSun := RenderShadow(gDateTime, $00885B00, 12, 180);

    // show the suns location on the earth
    Renderer.Font.Define('Arial', clRed, clWhite, 9, guPixel, 0, []);
		RenderTextout( ptSun, 'S' );

    // if not using the spherical projection show the hour values
		if Projector.ProjectionClass <> 'TGSphericalPrj' then
		begin
      Renderer.Font.Color := clWhite;
			RenderTextOut( PointLL( 0, GU_90_DEGREE - GU_DEGREE ), '0' );
			RenderTextOut( PointLL( 0, -GU_90_DEGREE + GU_DEGREE * 10 ), '0' );

			for idx := 1 to 11 do
			begin
				RenderTextOut( PointLL( GU_DEGREE * idx * 15, GU_90_DEGREE - GU_DEGREE ), '+' + IntToStr( idx ));
				RenderTextOut( PointLL( GU_DEGREE * idx * -15, GU_90_DEGREE - GU_DEGREE ), '-' + IntToStr( idx ));
				RenderTextOut( PointLL( GU_DEGREE * idx * 15, -GU_90_DEGREE + GU_DEGREE * 10 ), '+' + IntToStr( idx ));
				RenderTextOut( PointLL( GU_DEGREE * idx * -15, -GU_90_DEGREE + GU_DEGREE * 10 ), '-' + IntToStr( idx ));
			end;
			RenderTextOut( PointLL( -GU_180_DEGREE, GU_90_DEGREE - GU_DEGREE ), '-12');
			RenderTextOut( PointLL( -GU_180_DEGREE, -GU_90_DEGREE + GU_DEGREE * 10 ), '-12');
		end;
	end;
end;

{----------------------------------------------------------------------------}
procedure TForm1.GlobePaint(Sender: TObject);
begin
	Caption := TITLE + DateTimeToStr( gDateTime );
end;

{----------------------------------------------------------------------------}
procedure TForm1.Timer1Timer(Sender: TObject);
{$WRITEABLECONST ON}
const
	iCounter : integer = 0;
{$WRITEABLECONST OFF}
begin
	if not cbxAuto.Checked then
		iCounter := 0
	else
	begin
		gDateTime := Now;

    // adjust the slider bars to reflect the current time.
		Dec( iCounter );
		if iCounter <= 0 then
		begin
			iCounter := 5;
			sbTime.Position := Round( Frac( gDateTime ) * 1440 );
			sbDate.Position := Trunc( Date ) - Trunc( gFirstDayOfYear );
			cbxAuto.Checked := True;
			Globe.RedrawLayers;
			gDateTime := Now;
		end;
	end;
end;

{----------------------------------------------------------------------------}
procedure TForm1.sbDateChange(Sender: TObject);
begin
	cbxAuto.Checked := False;
	gDateTime := gFirstDayOfYear + sbDate.Position + sbTime.Position / 1440;
	Globe.RedrawLayers;
end;

{----------------------------------------------------------------------------}
procedure TForm1.GlobePaintGraticule(Sender: TObject);
var
	idx, iStep  : Integer;
begin
  // Renders a user defined graticule on the globe.
	with Globe do
	begin
    Renderer.Pen.Define( clWhite, gpsDot, 1, guPixel);

		iStep := MaxVal(36, MinVal(180, Round(180 * Globe.Projector.ScaleFactor)));

		{ draw lines of Longitude }
		idx := -GU_180_DEGREE + 1;
		repeat
			RenderLine(PointLL(idx, -GU_90_DEGREE + GU_DEGREE), PointLL(idx, GU_90_DEGREE - GU_DEGREE), iStep);
			Inc(idx, 15 * GU_DEGREE);
		until idx >= GU_180_DEGREE;

		{ draw lines of Latitude }
		idx := 0;
		repeat
			RenderLine(PointLL(GU_180_DEGREE, idx), PointLL( -GU_180_DEGREE, idx), iStep * 2);
			RenderLine(PointLL(GU_180_DEGREE, -idx), PointLL( -GU_180_DEGREE, -idx), iStep * 2);
			Inc(idx, 15 * GU_DEGREE);
		until idx >= 85 * GU_DEGREE;
	end;
end;

{----------------------------------------------------------------------------}
procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
	Globe.Projector.ProjectionClass := 'TGSphericalPrj';
	Globe.Projector.ScaleFactor := 0;
end;

{----------------------------------------------------------------------------}
procedure TForm1.SpeedButton3Click(Sender: TObject);
begin
	Globe.Projector.ProjectionClass := 'TGCartesianPrj';
	Globe.Projector.ScaleFactor := 0;
end;

{----------------------------------------------------------------------------}
procedure TForm1.GlobeMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // zooms the globe about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.0 / 1.1 );
end;

{----------------------------------------------------------------------------}
procedure TForm1.GlobeMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  // zooms the globe about the current mouse location
  Globe.Projector.AdjustScaleFactor( Globe.ScreenToClient(MousePos), 1.1 );
end;

end.
